@@ -49,7 +49,7 @@ extern wm_functions_t wayfire_functions;
 /*----------------------------------------------------------------------------*/
 
 static GtkBuilder *builder;
-static GtkWidget *da, *main_dlg, *undo, *zin, *zout, *conf, *clbl, *cpb, *ident;
+static GtkWidget *da, *main_dlg, *undo, *zin, *zout, *conf, *clbl, *cpb, *ident, *overlay, *zooms;
 static GtkWidget *id[MAX_MONS], *lbl[MAX_MONS];
 
 monitor_t mons[MAX_MONS];
@@ -100,9 +100,10 @@ static void find_touchscreens (void);
 static void find_backlights (void);
 static int get_backlight (int mon);
 static void set_backlight (int mon, int level);
-static void button_press_event (GtkWidget *, GdkEventButton *ev, gpointer);
+static gboolean button_press_event (GtkWidget *, GdkEventButton *ev, gpointer);
 static gboolean motion_notify_event (GtkWidget *da, GdkEventMotion *ev, gpointer);
-static void button_release_event (GtkWidget *, GdkEventButton *, gpointer);
+static gboolean button_release_event (GtkWidget *, GdkEventButton *, gpointer);
+static gboolean scroll (GtkWidget *, GdkEventScroll *ev, gpointer);
 static void gesture_pressed (GtkGestureLongPress *, gdouble x, gdouble y, gpointer);
 static void gesture_end (GtkGestureLongPress *, GdkEventSequence *, gpointer);
 static void handle_apply (GtkButton *, gpointer);
@@ -809,7 +810,7 @@ static void set_backlight (int mon, int level)
 /* Event handlers */
 /*----------------------------------------------------------------------------*/
 
-static void button_press_event (GtkWidget *, GdkEventButton *ev, gpointer)
+static gboolean button_press_event (GtkWidget *, GdkEventButton *ev, gpointer)
 {
     GtkWidget *menu;
     int m;
@@ -834,6 +835,7 @@ static void button_press_event (GtkWidget *, GdkEventButton *ev, gpointer)
         curmon = -1;
         gtk_menu_popup_at_pointer (GTK_MENU (menu), gtk_get_current_event ());
     }
+    return TRUE;
 }
 
 static gboolean motion_notify_event (GtkWidget *da, GdkEventMotion *ev, gpointer)
@@ -865,9 +867,17 @@ static gboolean motion_notify_event (GtkWidget *da, GdkEventMotion *ev, gpointer
     return FALSE;
 }
 
-static void button_release_event (GtkWidget *, GdkEventButton *, gpointer)
+static gboolean button_release_event (GtkWidget *, GdkEventButton *, gpointer)
 {
     curmon = -1;
+    return TRUE;
+}
+
+static gboolean scroll (GtkWidget *, GdkEventScroll *ev, gpointer)
+{
+    if (ev->direction == 0) handle_zoom (NULL, (gpointer) 1);
+    if (ev->direction == 1) handle_zoom (NULL, (gpointer) -1);
+    return TRUE;
 }
 
 static void gesture_pressed (GtkGestureLongPress *, gdouble x, gdouble y, gpointer)
@@ -1079,11 +1089,12 @@ static void init_config (void)
     curmon = -1;
     scale = 8;
     da = (GtkWidget *) gtk_builder_get_object (builder, "da");
-    gtk_widget_set_events (da, GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
+    gtk_widget_set_events (da, GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_SCROLL_MASK);
     g_signal_connect (da, "draw", G_CALLBACK (draw), NULL);
     g_signal_connect (da, "button-press-event", G_CALLBACK (button_press_event), NULL);
     g_signal_connect (da, "button-release-event", G_CALLBACK (button_release_event), NULL);
     g_signal_connect (da, "motion-notify-event", G_CALLBACK (motion_notify_event), NULL);
+    g_signal_connect (da, "scroll-event", G_CALLBACK (scroll), NULL);
 
     undo = (GtkWidget *) gtk_builder_get_object (builder, "btn_undo");
     g_signal_connect (undo, "clicked", G_CALLBACK (handle_undo), NULL);
@@ -1091,8 +1102,11 @@ static void init_config (void)
 
     zin = (GtkWidget *) gtk_builder_get_object (builder, "btn_zin");
     zout = (GtkWidget *) gtk_builder_get_object (builder, "btn_zout");
+    overlay = (GtkWidget *) gtk_builder_get_object (builder, "overlay");
+    zooms = (GtkWidget *) gtk_builder_get_object (builder, "zooms");
     g_signal_connect (zin, "clicked", G_CALLBACK (handle_zoom), (gpointer) 1);
     g_signal_connect (zout, "clicked", G_CALLBACK (handle_zoom), (gpointer) -1);
+    gtk_overlay_add_overlay (GTK_OVERLAY (overlay), zooms);
 
     g_signal_connect (gtk_builder_get_object (builder, "btn_apply"), "clicked", G_CALLBACK (handle_apply), NULL);
     g_signal_connect (gtk_builder_get_object (builder, "btn_menu"), "clicked", G_CALLBACK (handle_menu), NULL);
@@ -1157,6 +1171,11 @@ const char *tab_name (int tab)
     return C_("tab", "Screens");
 }
 
+const char *icon_name (int tab)
+{
+    return "video-display";
+}
+
 const char *tab_id (int tab)
 {
     return NULL;
